%-----------------------------------------------------
% Code used to simulate a 2D viscosity profile
%-----------------------------------------------------

clear all
close all

%initial velocities
u=zeros(70,108); %70 meshes vertically, 108 meshes horizontally

%The topographic profile is defined by an image
im_zones = imread('ESM5a_dome_shape_march2019.tif');
im_zones = double(im_zones);
im_zones = flipud(im_zones);

%number of lines and columns
[nrow, ncol]=size(u);

%viscosity - here constant
mu=u.*0+1e6;

%matrix where 0 = lava, 1 = air
zones=im_zones*0;
zones(im_zones>50)=1;

% mesh sizes
% x is oriented in the flow direction
% y is perpendicular
% z is normal to the ground
dx=0.5;dy=0.5; dz=0.5;

%dfinition of the normal axis
z = ( 1:size(u,1) )' * dz * ones(1,size(u,2)) -dz;

dt    = 1e-5;   %time step
rho   = 2300;   %density
alpha =    0;   %slope
g     = 9.79;   %gravity (m/s)

U=[]; %surface velocity

%Calculation of the stationnary equilibrium
%you can stop the code once it has converged
for t=0:dt:500000*dt;
    
    %viscous shear stress and force in y; T = mu*du/dy
    Tau_y = mu(:,2:end-1).*(  (u(:,3:end)-u(:,2:end-1) ).*zones(:,3:end)...
            +  (u(:,1:end-2)-u(:,2:end-1)).*zones(:,1:end-2) ) /dy;
    Tau_y=[zeros(size(Tau_y,1),1) Tau_y zeros(size(Tau_y,1),1)];
    Fy = Tau_y * dx*dz; %viscous force
    
    %viscous shear stress and force in z
    Tau_z(2:nrow-1, :) = mu(2:nrow-1, :) .*( (u(3:end,:)-u(2:end-1,:)).*zones(3:end,:)...
                         +  (u(1:end-2,:)-u(2:end-1,:)).*zones(1:end-2,:)  )/dz;
    Tau_z(1,:) = mu(1,:) .* (0-u(1,:))/dz;
    Tau_z(nrow,:) = mu(nrow,:) .* (u(end-1,:)-u(end,:))/dz;
    Fz = Tau_z * dx*dy; %viscous force
    
    %Driven pressure
    % **** case 1 : constant pressure difference downwards
    dP = 10;
    Fp(1:nrow, 1:ncol) = dP*dy*dz;
    
    % **** cas 2 : flow driven by its weight
    %Fp = dx*dy*dz*rho*g*sind(alpha);
    
    % **** cas 3 : Couette flow
    %Fp=0;
    %u(end,:)=1;
    
    %Newton's second law
    du_dt = ( Fp+Fy+Fz ) / (dx*dy*dz*rho);
    
    %new velocities
    u = u + du_dt*dt;
    
    %bonndary conditions
    u(:,1)=0; u(:,end)=0;   %null velocities at the vertical edges
    u(1, :)=0;              %null velocities at the base
    u(zones==0)=0;          %null velocity in the air
    
    %records the surface vlocity to check if a stationnary velocity is
    %reached
    if t/(10000*dt) == round(t/(10000*dt))
        U = [U, u(end,51)];
    end
    
    %plot of the velocity
    if t/(10000*dt) == round(t/(10000*dt))
        figure(3)
        plot( u(:,86), z(:,1));
        hold on
        drawnow
    end
    
end

%3D-view of the velocity profile
figure
[y,zz]=meshgrid(1:ncol,1:nrow);
surf(u,y,zz)
xlabel('velocity m/s'); ylabel('width (m)'); zlabel('thickness (m)')